;PED CADD                                                            Version 2.0
;
;By:  Technical Software, Inc.
;     Christopher Heltzel
;     4555 Emery Industrial Pkwy.
;     Suite 102
;     Cleveland, Ohio 44128
;     1-900-820-ACAD
;
;File Name:  BALLNOTE.LSP
;
;Contents:  C:BN, Add_note, Add_balloon, Create_balloon, Query_string, Query_int
;           and $error$.
;
;Usage:  Type BN.
;        Select points to define the leader line starting with the arrow end.
;        Press <RETURN> key to complete the leader.
;        Enter either P, D, S, or N corresponding to Plain, Detail, Standard, or
;           Note.
;
;        When P, D, or S are entered, the following is expected
;           If P, Enter the Item Number.
;           If D, Enter the Detail Number and then the Sheet Number.
;           If S, Enter the Item Number.
;        The routine will continue to prompt for a balloon type and loop though
;           the questions above, until you press <RETURN>.
;
;        When N is entered, you must enter U for Underline or press <RETURN> for
;           Regular text.
;        Enter the text string and press <RETURN> at the end of the first line.
;        Continue to enter succesive lines of text or press <RETURN> if the note
;           is complete.
;        If the note was created on the left side of the leader extension, you
;           have the option to left justify the text or keep it right justified.
;        Enter Y for left justification or N to retain right justification.
;        Sets snap to nearest and prompts for the arrow location.
;
;Results:     Creates a leader line with multiple bends and a horizontal
;          extension line at its end (layer D).  Either detail balloons or text
;          are added after the extension.
;             If one of the balloon options is selected then the appropriate
;          block is created if it is not already defined.  The geometry is on
;          layer 0 and text is on layer T.  All text is 0.125" high (or slightly
;          smaller to fill in balloon) multiplied by the dimscale.  The routine
;          will continue to add balloons that are strung together until you
;          exit.
;             If Note is selected you have the option of underlined or regular
;          text.  The routine allows you to add succesive lines, equally spaced.

(defun C:BN (/ #error# layer dscale osmode p1 p2 p3 p4 p5 a1 a2 op info_type)
  (setvar "cmdecho" 0)
  (command "undo" "e" "undo" "m")
  (setq #error# *error* *error* $error$                                         ;save system variables
        layer (getvar "clayer")
        dscale (getvar "dimscale")
        osmode (getvar "osmode")
        blipmode (getvar "blipmode")
  )
  (if (tblsearch "layer" "d")                                                   ;create layer if nonexistent
    (command "layer" "t" "d" "s" "d" "")
    (command "layer" "m" "d" "c" "2" "d" "l" "continuous" "d" "")
  )
  (setvar "osmode" 512)                                                         ;nearest snap mode
  (setvar "blipmode" 0)
  (graphscr)
  (setq p1 (getpoint "\nSelect leader arrow location: "))
  (setvar "osmode" 0)
  (setq p2 (getpoint p1 "\nSecond point of leader line: "))
  (command "line" p1 p2 "")
  (setq a1 (angle p1 p2)                                                        ;determine angle for arrowhead
        a2 a1                                                                   ;set initial angle to determine ext. line direction
  )
  (while (setq p3 (getpoint p2 "\nNext point of leader line or <RETURN>: "))    ;loop to add multiple bends in leader
    (command "line" p2 p3 "")
    (setq a2 (angle p2 p3)                                                      ;set angle to determine extension line direction
          p2 p3
  ) )
  (if (and (> a2 (/ pi 2)) (< a2 (* pi 1.5)))
    (setq op '-)                                                                ;This section determines wheather the extension
    (setq op '+)                                                                ; line goes to the right or to the left.
  )
  (command "line" p2 (list ((eval op) (car p2) (* 0.25 dscale)) (cadr p2)) "")  ;add extension line
  (setq p4
    (strcat "@" (rtos (* 0.1875 dscale) 2 4) "<" (rtos (/ (* a1 180) pi) 2 4))  ;relative polar coord for arrowhead
  )
  (command "pline" p1 "h" "0.0" (* 0.0312 dscale) p4 "")                        ;arrowhead
  (initget 1 "Plain Detail Standard Note")
  (setq info_type (getkword "\nEnter (Plain/Detail/Standard/Note): "))
  (if (= info_type "Note")
    (Add_note)
    (Add_balloon)
  )
  (command "layer" "s" layer "")                                                 ;resets variables
  (setvar "blipmode" blipmode)
  (setvar "osmode" osmode)
  (setvar "highlight" 1)
  (setq *error* #error#)                                                        ;reset error routine
  (princ)
)

(defun Add_note (/ text_size text_prefix ls text_string just n x tmp ename)
  (setq p5 (list ((eval op) (car p2) (* 0.375 dscale)) (- (cadr p2)             ;text insert point
    (* 0.0625 dscale)))
  )
  (if (tblsearch "layer" "t")                                                   ;create layer if nonexistent
    (command "layer" "t" "t" "s" "t" "")
    (command "layer" "m" "t" "c" "9" "t" "l" "continuous" "t" "")
  )
  (setq text_size (rtos (* 0.125 dscale)))
  (initget "Underline Regular")
  (setq text_prefix (getkword "Note type (Underline/<Regular>): "))
  (if (= text_prefix "Underline")
    (setq text_prefix "%%U")
    (setq text_prefix "")
  )
  (setq ls (list nil))                                                          ;initialize list of string entities
  (setvar "blipmode" 1)                                                         ;blips to visually mark text insertion points
  (while (/= (setq text_string
             (strcase (getstring T "Enter text string or <exit>: ")))
         "")
    (setq text_string (strcat text_prefix text_string))
    (if (= op '+)
      (command "text" p5 text_size "0" text_string)                             ;left or right justified
      (command "text" "r" p5 text_size "0" text_string)
    )
    (setq ls (cons (entlast) ls))                                               ;add last text entity to list
    (setq p5 (list (car p5) (- (cadr p5) (* 0.25 dscale))))                     ;reset insertion point down one line
  )
  (setvar "blipmode" 0)
  (if (= op '-)                                                                 ;right justified text can be moved to left just.
    (progn
      (initget 1 "Yes No")
      (setq just (getkword "Do you want left justification (Y/N): ")
            n 0
            x 0
      )
      (if (= just "Yes")
        (progn
          (While (setq ename (nth n ls))                                        ;loops through text entity list
            (setq tmp_x (car (cdr (assoc 10 (entget ename))))                   ;sets tmp_x to x coord of text entity
                  n (1+ n)
            )
            (if (or (< tmp_x x) (= x 0))                                        ;sets x to smallest x corrd value
              (setq x tmp_x)
          ) )
          (setq n 0)
          (while (setq ename (nth n ls))                                        ;loops through text entity list
            (setq p1 (cdr (assoc 10 (entget ename)))                            ;sets p1 to x corrd value
                  n (1+ n)
            )
            (command "move" ename "" p1 (list x (cadr p1)))                     ;moves text entity to new x position
) ) ) ) ) )

(defun Add_balloon (/ balloon_blk det sht)
  (setq p5 (list ((eval op) (car p2) (* 0.5 dscale)) (cadr p2)))            ;sets insert point for first balloon
  (while (and info_type (/= info_type "Exit"))
    (if (= info_type "Plain")
      (setq balloon_blk "p_balloon")
      (setq balloon_blk "ds_balloon")
    )
    (if (not (tblsearch "block" balloon_blk))                                   ;creates the balloon block if not already defined
      (Create_balloon)
    )
    (if (= info_type "Plain")                                                   ;prompts for proper information based on balloon
      (progn                                                                    ; type.
        (setq det (Query_string 1 3 "\nItem Number: "))                         ; inserts the proper balloon block.
        (command "insert" balloon_blk p5 (rtos dscale) "" "0" det)
      )
      (progn
        (if (= info_type "Detail")
          (setq det (Query_int 1 999 "\nDetail Number: ")
                sht (Query_int 1 999 "\nSheet Number: ")
          )
          (setq det (Query_int 1 999 "\nItem Number: ")
                sht "STD"
        ) )
        (command "insert" balloon_blk p5 (rtos dscale) "" "0" det sht)
    ) )
    (initget "Plain Detail Standard Exit")
    (setq info_type (getkword "\nBalloon type (Plain/Detail/Standard/<Exit>): "));askes to add another balloon
    (setq p5 (list ((eval op) (car p5) (* 0.50 dscale)) (cadr p5)))            ;sets next block insertion point
) )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;(Create_balloon)
;Creates the balloon type described by the variable balloon_blk

(defun Create_balloon (/ ss1 sty)
  (princ "\nPlease wait...")
  (setvar "aflags" 0)
  (setq ss1 (ssadd))
  (command "circle" "0,0" "0.25")
  (ssadd (entlast) ss1)
  (command "line" '(-0.25 0) '(0.25 0) "")
  (setq sty (getvar "textstyle"))
  (if (tblsearch "style" "s")
    (command "style" "s" "" "" "" "" "" "" "")
    (command "style" "s" "simplex" "0" ".9" "0" "n" "n" "n")
  )
  (if (tblsearch "layer" "t")                                                   ;create layer if nonexistent
    (command "layer" "t" "t" "s" "t" "")
    (command "layer" "m" "t" "c" "9" "t" "l" "continuous" "t" "")
  )
  (if (= info_type "Plain")
    (progn
      (command "erase" (entlast) "")
      (command "attdef" "" "ITEM" "Item Number" "" "m" "0,0" "0.125" "0")
      (prompt "\n \n \n")
      (ssadd (entlast) ss1)
    )
    (progn
      (ssadd (entlast) ss1)
      (command "attdef" "" "DET" "Detail Number" "" "c" '(0 0.0625) "0.10" "0")
      (ssadd (entlast) ss1)
      (command "attdef" "" "SHT" "Sheet Number" "" "c" '(0 -0.1625) "0.10" "0")
      (prompt "\n \n \n")
      (ssadd (entlast) ss1)
  ) )
  (command "block" balloon_blk "0,0" ss1 ""
           "style" sty "" "" "" "" "" "" ""
           "layer" "s" "d" ""
) )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;(Query_string <number> <number> <string>)
;Retuns a text string input by the user.
;The two <number> parameters are the minimum and maximum number of characters
;respectively.  The <string> is the prompt to the user.

(defun Query_string (lower_lim upper_lim prmpt / strng)
  (setq strng (strcase (getstring 1 prmpt)))
  (while (or (< (strlen strng) lower_lim) (> (strlen strng) upper_lim))
    (setq strng (strcase (getstring 1 (strcat "Must be between "
      (itoa lower_lim) " and " (itoa upper_lim) " characters.\n" prmpt)))
  ) )
  (setq strng strng)
)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;(Query_int <number> <number> <string>)
;Retuns an integer value input by the user.
;The two <number> parameters are the minimum and maximum input values
;respectively.  The <string> is the prompt to the user.

(defun Query_int (lower_lim upper_lim prmpt / strng)
  (setq strng (getint prmpt))
  (while (or (< strng lower_lim) (> strng upper_lim))
    (setq strng (getint (strcat "Must be a number between "
      (itoa lower_lim) " and " (itoa upper_lim) ".\n" prmpt))
  ) )
  (setq strng strng)
)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;(*error* <string>)
;Returns the system error message <msg>
;UNDO BACK is performed to the last marker set by the routine in error.

(defun $error$ (msg)
  (princ (strcat "\nError: " msg))
  (setvar "highlight" 1)
  (setq *error* #error#)
  (princ)
)

(prompt "\nC:BN")
(princ)
